## This script provedes analysis of standardized aerial survey data provided in
## a single feature class. The aerial survey data must be standardized according
## to the US Forest Service Forest Health Monitoring Aerial Survey Codes
## (Appendix E) and the US Forest Service Aerial Detection Survey GIS Handbook.
## The user is asked to select either a damage agent or damage type for the
## analysis. The tool calculates the number of times a particular polygon has
## been damaged by the given agent or type. It also counts the Mean and Sum of
## severity of the damage and the duration of the damage. These calculations
## are added as fields to the attribute table. The output of this tool is a
## shapefile that includes the new attributes.

import arcpy
from arcpy import env
import string
import sys
import os, stat

#These codes are generated by the damage agent code generator tool/script and
#can be found in the file code_list.txt that is generated as the output from
#that tool. This is the dictionary of damage agents that the tool uses to compare
#the input from the user and perform the analysis. 
codes = {'71000-harvest': 71000, '50020-saltwater injury - flooding / hurricane': 50020, '50016-mud-land slide': 50016, '50006-hail': 50006, '12197-winter moth': 12197, '22006-black knot of cherry': 22006, '25001-blight': 25001, '12098-leafcutting bees': 12098, '50800-other abiotic damage (known)': 50800, '15001-shoot borers': 15001, '25057-Sirococcus tip blight': 25057, '25029-hardwood anthracnose': 25029, '25010-sycamore anthracnose': 25010, '15055-pine gall weevil': 15055, '25072-sirococcus shoot blight': 25072, '15004-bronze birch borer': 15004, '14028-oystershell scale': 14028, '50000-abiotic damage': 50000, '12037-large aspen tortrix': 12037, '50005-frost': 50005, '80000-multi-damage (insect/disease)': 80000, '24011-larch decline': 24011, '90010-dieback': 90010, '85000-invasive plants': 85000, '12119-locust leafminer': 12119, '17003-Cooley spruce gall adelgid': 17003, '12006-skeletonizer': 12006, '90001-broken top': 90001, '14009-Saratoga spittlebug': 14009, '12120-Bruce spanworm': 12120, '23003-vine damage': 23003, '12055-oak leaftier': 12055, '10000-general insects': 10000, '60000-competition': 60000, '22023-chestnut blight': 22023, '12038-spruce budworm': 12038, '22011-Caliciopsis canker': 22011, '17005-oak gall wasp': 17005, '90002-dead top': 90002, '15052-ambrosia beetles': 15052, '12125-spring cankerworm': 12125, '24028-hemlock decline': 24028, '12147-maple leafroller': 12147, '25000-foliage and shoot diseases': 25000, '70005-land clearing': 70005, '90000-unknown': 90000, '12083-hemlock looper': 12083, '90008-foliage discoloration': 90008, '14002-western larch woolly aphid': 14002, "24003-Stillwell's syndrome": 24003, '11010-eastern larch beetle': 11010, '12156-willow defoliation': 12156, '12139-larch sawfly': 12139, '11009-spruce beetle': 11009, '50003-drought': 50003, '11003-southern pine beetle': 11003, '22040-sterile conk trunk rot of birch': 22040, '12015-alder flea beetle': 12015, '25055-Septoria leaf spot of alder': 25055, '90009-mortality': 90009, '22026-Cytospora canker of fir': 22026, '50900-unknown abiotic damage': 50900, '13012-periodical cicada': 13012, '12070-birch leafminer': 12070, '12082-fall webworm': 12082, '41002-American beaver': 41002, '25058-Diplodia canker': 25058, '12002-leaftier': 12002, '25048-ash rust': 25048, '25075-tar spot': 25075, '25043-Swiss needle cast': 25043, '24000-decline complexes/dieback': 24000, '14000-sap feeding insects': 14000, '12223-bare-patched oak leafroller': 12223, '15086-Sitka spruce weevil': 15086, '26000-stem rust': 26000, '22042-beech bark disease': 22042, '25067-spruce needle cast': 25067, '41010-woodpeckers': 41010, '50009-nutrient imbalances': 50009, '24009-fall hardwood defoliator complex': 24009, '12086-satin moth': 12086, '21000-root/butt diseases': 21000, '24005-birch dieback': 24005, '12136-yellowheaded spruce sawfly': 12136, '24900-unknown decline/complex/wilt': 24900, '10002-pine tip moths': 10002, '12000-defoliators': 12000, '11030-ips engraver beetles': 11030, '15050-Engelmann spruce weevil': 15050, '25065-spruce needle rust': 25065, '24022-Dutch elm disease': 24022, '15065-Nantucket pine tip moth': 15065, '17002-eastern spruce gall adelgid': 17002, '26001-white pine blister rust': 26001, '14003-balsam woolly adelgid': 14003, '70001-herbicides': 70001, '17000-gallmaker insects': 17000, '25035-Lophodermium needle cast of pines': 25035, '12167-striped alder sawfly': 12167, '13006-cicadas': 13006, '25056-Septoria leaf spot and canker': 25056, '22000-stem decays/cankers': 22000, '26002-western gall rust': 26002, '14016-beech scale': 14016, '50007-heat': 50007, '25800-other foliage / shoot disease (known)': 25800, '70000-human activities': 70000, '14033-red pine scale': 14033, '14046-pine leaf adelgid': 14046, '24008-decline complex': 24008, '13029-eastern pine weevil': 13029, '25036-Marssonina blight': 25036, '12005-sawflies': 12005, '70012-suppression': 70012, '12215-imported willow leaf beetle': 12215, '25005-needlecast': 25005, '12047-larch casebearer': 12047, '11011-black turpentine beetle': 11011, '24013-maple decline': 24013, '12089-gypsy moth': 12089, '99999-no data': 99999, '70014-road salt or deicers': 70014, '12174-pine looper': 12174, '50001-air pollutants': 50001, '70004-improper planting technique': 70004, '50008-lightning': 50008, '12800-other defoliator (known)': 12800, '25025-anthracnose': 25025, '12138-Japanese beetle': 12138, '12024-oak leafroller': 12024, '25059-leaf blister of oak': 25059, '22030-Eutypella canker': 22030, '12127-maple leafcutter': 12127, '15039-locust borer': 15039, '70011-soil compaction': 70011, '12068-browntail moth': 12068, '41006-common porcupine': 41006, '70006-land use conversion': 70006, '50002-chemical': 50002, '24026-bacterial leaf scorch': 24026, '15090-sirex woodwasp': 15090, '25076-birch leaf fungus': 25076, '12069-pine needleminer': 12069, '24002-Norfolk Island pine decline': 24002, '15087-emerald ash borer': 15087, '50014-winter injury': 50014, '12081-cherry scallop shell moth': 12081, '25052-Rhizosphaera needle cast': 25052, '12014-fall cankerworm': 12014, '22038-Hypoxylon canker of aspen': 22038, '14020-elongate hemlock scale': 14020, '12008-spanworm': 12008, '25068-hardwood leaf rusts': 25068, '12067-linden looper': 12067, '12019-orangestriped oakworm': 12019, '12065-maple trumpet skeletonizer': 12065, '50004-flooding-high water': 50004, '21012-tomentosus root rot/false velvet top fungus': 21012, '12110-white pine sawfly': 12110, '12096-forest tent caterpillar': 12096, '25032-fir needle cast': 25032, '15088-hemlock borer': 15088, '12074-spotted tussock moth': 12074, '15022-Zimmerman pine moth': 15022, '30004-ground fire damage': 30004, '12079-saddled prominent': 12079, '15031-sugar maple borer': 15031, '14042-woolly alder aphid': 14042, '22089-apple scab': 22089, '24025-Verticillium wilt': 24025, '25054-brown spot needle blight': 25054, '11000-bark beetles': 11000, '50011-snow-ice': 50011, '15058-carpenterworm': 15058, '50013-wind-tornado/hurricane': 50013, '24014-oak decline': 24014, '25900-unknown foliage / shoot disease': 25900, '12190-hickory tussock moth': 12190, '12003-loopers': 12003, '30000-fire': 30000, '12076-hesperiid moth': 12076, '14029-pinyon needle scale': 14029, '14004-hemlock woolly adelgid': 14004, '15005-twolined chestnut borer': 15005, '12141-elm leaf beetle': 12141, '12004-needleminers': 12004, '24007-pest complex': 24007, '17024-crypt gall wasp': 17024, '12026-arborvitae leafminer': 12026, '12080-variable oakleaf caterpillar': 12080, '70007-logging damage': 70007, '12900-unknown defoliator': 12900, '24004-ash yellows disease': 24004, '12100-early birch leaf edgeminer': 12100, '80003-five-needle pine decline': 80003, '41004-mice or voles': 41004, '80001-aspen defoliation': 80001, '12029-oak skeletonizer': 12029, '22043-Nectria canker': 22043, '12010-pine false webworm': 12010, '25033-white pine needle cast': 25033, '12193-pitch pine looper': 12193, '17800-other gallmaking insect (known)': 17800, '12084-oak looper': 12084, '15000-boring insects': 15000, '14058-pear thrips': 14058, '22900-unknown stem decay / canker disease': 22900, '70010-roads': 70010, '30002-human caused fire': 30002, '21001-Armillaria root disease': 21001, '12041-jack pine budworm': 12041, '24800-other decline/complex/wilt (known)': 24800, '30001-wild fire': 30001, '25004-leaf spots': 25004, '11012-red turpentine beetle': 11012, '12093-eastern tent caterpillar': 12093, '16000-seed/cone/flower/fruit insects': 16000, '25077-Septoria leaf spot of maple': 25077}
#this is the damage agent list that goes into the tool in Arc for the
#dropdown menu. It is generated from the damage agent code generator
#tool and can be found in the code_list.txt file that is the output
dmg_agent_list=['10000-general insects', '10002-pine tip moths', '11000-bark beetles', '11003-southern pine beetle', '11009-spruce beetle', '11010-eastern larch beetle', '11011-black turpentine beetle', '11012-red turpentine beetle', '11030-ips engraver beetles', '12000-defoliators', '12002-leaftier', '12003-loopers', '12004-needleminers', '12005-sawflies', '12006-skeletonizer', '12008-spanworm', '12010-pine false webworm', '12014-fall cankerworm', '12015-alder flea beetle', '12019-orangestriped oakworm', '12024-oak leafroller', '12026-arborvitae leafminer', '12029-oak skeletonizer', '12037-large aspen tortrix', '12038-spruce budworm', '12041-jack pine budworm', '12047-larch casebearer', '12055-oak leaftier', '12065-maple trumpet skeletonizer', '12067-linden looper', '12068-browntail moth', '12069-pine needleminer', '12070-birch leafminer', '12074-spotted tussock moth', '12076-hesperiid moth', '12079-saddled prominent', '12080-variable oakleaf caterpillar', '12081-cherry scallop shell moth', '12082-fall webworm', '12083-hemlock looper', '12084-oak looper', '12086-satin moth', '12089-gypsy moth', '12093-eastern tent caterpillar', '12096-forest tent caterpillar', '12098-leafcutting bees', '12100-early birch leaf edgeminer', '12110-white pine sawfly', '12119-locust leafminer', '12120-Bruce spanworm', '12125-spring cankerworm', '12127-maple leafcutter', '12136-yellowheaded spruce sawfly', '12138-Japanese beetle', '12139-larch sawfly', '12141-elm leaf beetle', '12147-maple leafroller', '12156-willow defoliation', '12167-striped alder sawfly', '12174-pine looper', '12190-hickory tussock moth', '12193-pitch pine looper', '12197-winter moth', '12215-imported willow leaf beetle', '12223-bare-patched oak leafroller', '12800-other defoliator (known)', '12900-unknown defoliator', '13006-cicadas', '13012-periodical cicada', '13029-eastern pine weevil', '14000-sap feeding insects', '14002-western larch woolly aphid', '14003-balsam woolly adelgid', '14004-hemlock woolly adelgid', '14009-Saratoga spittlebug', '14016-beech scale', '14020-elongate hemlock scale', '14028-oystershell scale', '14029-pinyon needle scale', '14033-red pine scale', '14042-woolly alder aphid', '14046-pine leaf adelgid', '14058-pear thrips', '15000-boring insects', '15001-shoot borers', '15004-bronze birch borer', '15005-twolined chestnut borer', '15022-Zimmerman pine moth', '15031-sugar maple borer', '15039-locust borer', '15050-Engelmann spruce weevil', '15052-ambrosia beetles', '15055-pine gall weevil', '15058-carpenterworm', '15065-Nantucket pine tip moth', '15086-Sitka spruce weevil', '15087-emerald ash borer', '15088-hemlock borer', '15090-sirex woodwasp', '16000-seed/cone/flower/fruit insects', '17000-gallmaker insects', '17002-eastern spruce gall adelgid', '17003-Cooley spruce gall adelgid', '17005-oak gall wasp', '17024-crypt gall wasp', '17800-other gallmaking insect (known)', '21000-root/butt diseases', '21001-Armillaria root disease', '21012-tomentosus root rot/false velvet top fungus', '22000-stem decays/cankers', '22006-black knot of cherry', '22011-Caliciopsis canker', '22023-chestnut blight', '22026-Cytospora canker of fir', '22030-Eutypella canker', '22038-Hypoxylon canker of aspen', '22040-sterile conk trunk rot of birch', '22042-beech bark disease', '22043-Nectria canker', '22089-apple scab', '22900-unknown stem decay / canker disease', '23003-vine damage', '24000-decline complexes/dieback', '24002-Norfolk Island pine decline', "24003-Stillwell's syndrome", '24004-ash yellows disease', '24005-birch dieback', '24007-pest complex', '24008-decline complex', '24009-fall hardwood defoliator complex', '24011-larch decline', '24013-maple decline', '24014-oak decline', '24022-Dutch elm disease', '24025-Verticillium wilt', '24026-bacterial leaf scorch', '24028-hemlock decline', '24800-other decline/complex/wilt (known)', '24900-unknown decline/complex/wilt', '25000-foliage and shoot diseases', '25001-blight', '25004-leaf spots', '25005-needlecast', '25010-sycamore anthracnose', '25025-anthracnose', '25029-hardwood anthracnose', '25032-fir needle cast', '25033-white pine needle cast', '25035-Lophodermium needle cast of pines', '25036-Marssonina blight', '25043-Swiss needle cast', '25048-ash rust', '25052-Rhizosphaera needle cast', '25054-brown spot needle blight', '25055-Septoria leaf spot of alder', '25056-Septoria leaf spot and canker', '25057-Sirococcus tip blight', '25058-Diplodia canker', '25059-leaf blister of oak', '25065-spruce needle rust', '25067-spruce needle cast', '25068-hardwood leaf rusts', '25072-sirococcus shoot blight', '25075-tar spot', '25076-birch leaf fungus', '25077-Septoria leaf spot of maple', '25800-other foliage / shoot disease (known)', '25900-unknown foliage / shoot disease', '26000-stem rust', '26001-white pine blister rust', '26002-western gall rust', '30000-fire', '30001-wild fire', '30002-human caused fire', '30004-ground fire damage', '41002-American beaver', '41004-mice or voles', '41006-common porcupine', '41010-woodpeckers', '50000-abiotic damage', '50001-air pollutants', '50002-chemical', '50003-drought', '50004-flooding-high water', '50005-frost', '50006-hail', '50007-heat', '50008-lightning', '50009-nutrient imbalances', '50011-snow-ice', '50013-wind-tornado/hurricane', '50014-winter injury', '50016-mud-land slide', '50020-saltwater injury - flooding / hurricane', '50800-other abiotic damage (known)', '50900-unknown abiotic damage', '60000-competition', '70000-human activities', '70001-herbicides', '70004-improper planting technique', '70005-land clearing', '70006-land use conversion', '70007-logging damage', '70010-roads', '70011-soil compaction', '70012-suppression', '70014-road salt or deicers', '71000-harvest', '80000-multi-damage (insect/disease)', '80001-aspen defoliation', '80003-five-needle pine decline', '85000-invasive plants', '90000-unknown', '90001-broken top', '90002-dead top', '90008-foliage discoloration', '90009-mortality', '90010-dieback', '99999-no data'];
#the list of possible damage types. This list is takent from
#the Aerial Detection Survey GIS handbook. 
dmg_type_list=["Defoliation","Dieback","Mortality (Current Year)","Branch Breakage", "Discoloration",
               "Topkill", "Main Stem Broken/Uprooted", "Branch Flagging","No Damage", "Other Damage",
               "Previously Undocumented (Old) Mortality"]
##user defined variables
survey = arcpy.GetParameterAsText(0); # aerial survey data feature class 
folder = arcpy.GetParameterAsText(1); # folder for output shapefile
dmg_type = arcpy.GetParameterAsText(2); # damage type user would like to analize. If left empty it will search all.
damage_agent = arcpy.GetParameterAsText(3); # damage agent user would like to analize. If left empty it will search all. 

##Set the type code based on user input
if dmg_type == "Defoliation":
    type = 1;
elif dmg_type == "Mortality (Current Year)":
    type = 2;
    dmg_type="Mortality";
elif dmg_type == "Discoloration":
    type = 3;
elif dmg_type == "Dieback":
    type = 4;
elif dmg_type == "Topkill":
    type = 5;
elif dmg_type == "Branch Breakage":
    type = 6;
    dmg_type = "BranchBreak";
elif dmg_type == "Main Stem Broken/Uprooted":
    type = 7;
    dmg_type = "MainStemBroken";
elif dmg_type == "Branch Flagging":
    type = 8;
    dmg_type = "BranchFlag";
elif dmg_type == "No Damage":
    type = 9;
    dmg_type = "NoDamage";
elif dmg_type == "Other Damage":
    type = 10;
    dmg_type = "Other";
elif dmg_type == "Previously Undocumented (Old) Mortality":
    type = 11;
    dmg_type = "OldMortality";
else:
    type ="";
    
##match user input to code dictionary and get damage agent code
inList=False;
for k,v in codes.items():
    if damage_agent==k:
        dmg_agnt=str(v);
        inList=True;
    
#if code is not found in dictionary, check to see if user entered code
#if not make the damage agent an empty string (search all)
if inList == False :
    try:
        int (damage_agent);
        dmg_agnt = damage_agent;
    except:
        dmg_agnt = '';    
      
##Create Search string according to user input. Include case in which
##user does not select any damage type or damage agent
search_type = "DMG_TYPE1="+str(type);
flds=arcpy.ListFields(survey)
if "DCA2" in flds:
    search_agnt = "DCA1="+dmg_agnt+" OR DCA2="+dmg_agnt;
else:
    search_agnt = "DCA1="+dmg_agnt;
if type != "" and dmg_agnt != "":
    search = search_type+ " AND " +search_agnt;
elif type == "" and dmg_agnt != "":
    search = search_agnt;
elif dmg_agnt == "" and type != "":
    search = search_type;
elif dmg_agnt == "" and type == "":
    search = "RPT_YR > 0";
if dmg_type == '':
    dmg_type="AllTypes";
if damage_agent=='':
    dmg_agnt="AllAgents";


##Make temporary processing folders
arcpy.env.overwriteOutput = True;
temp=arcpy.CreateFileGDB_management(folder,"Scratch.gdb", "CURRENT");
arcpy.env.workspace=folder+"\Scratch.gdb";

##make the selection from the servey and make a new file, union the polygons and then repair the geometry to prevent errors
arcpy.MakeFeatureLayer_management(survey,"damaged_areas", search);
arcpy.Union_analysis("damaged_areas","dmg_areas_union", "ONLY_FID"); 
arcpy.RepairGeometry_management("dmg_areas_union")

##Make individual polygons from the Unioned features. 
arcpy.MultipartToSinglepart_management("dmg_areas_union", "dmg_areas");

##Find Duplicate polygons and give all duplicates the same ID and make this a new feature class
arcpy.FindIdentical_management("dmg_areas","dmg_duplicates","SHAPE")
lyr = arcpy.MakeFeatureLayer_management("dmg_areas", "dmg_areas_lyr")
tbl = arcpy.MakeTableView_management("dmg_duplicates","dmg_dup_tbl")
arcpy.AddJoin_management("dmg_areas_lyr","OBJECTID","dmg_dup_tbl","IN_FID")
arcpy.FeatureClassToFeatureClass_conversion("dmg_areas_lyr",temp, "dmg_areas_keys")
arcpy.CopyFeatures_management("dmg_areas_lyr","dmg_areas_keys")
arcpy.Delete_management(lyr);
arcpy.Delete_management(tbl);

#delete identicle polygons in the union from above steps
arcpy.DeleteIdentical_management("dmg_areas_keys","dmg_duplicates_FEAT_SEQ")

##Spatial Join the union result to the selection result and get the union count.
arcpy.SpatialJoin_analysis("dmg_areas_keys","damaged_areas","dmg_attributes", "JOIN_ONE_TO_MANY", "", "", "WITHIN");
arcpy.DeleteField_management("dmg_attributes", "Join_Count");

##Add fields for the new attributes, accumulated severity, average severity, start and end years of damage
## and the average number of years between damage occurances
arcpy.AddField_management("dmg_attributes","SUM_SEVERITY","SHORT")
arcpy.AddField_management("dmg_attributes","MEAN_SEVERITY","SHORT")
arcpy.AddField_management("dmg_attributes","START_YR","SHORT")
arcpy.AddField_management("dmg_attributes","END_YR","SHORT")
arcpy.AddField_management("dmg_attributes","AVG_YRS_BTWN_DMG","SHORT")

##get fields from the original dataset
fields = []
for fld in arcpy.ListFields("dmg_attributes","*"):
    fields.append(fld.name)

##Dictionary to hold data for each polygon ID (if duplicate polygons, all info will be stored in one entry)
stats={}

##Go through table and group the severity and year attributes for all of the unique polygon IDs in dictionary
with arcpy.da.UpdateCursor("dmg_attributes",["dmg_duplicates_FEAT_SEQ","SEVERITY1","RPT_YR"]) as rows:
    for row in rows:
        if row[0] in stats:
            if row[0] is not None and row[1]!= -1 and row[1] != "" and row[1] != 0 and row[1] != None:
                stats[row[0]]["SEVERITY"].append(row[1])
            if row[0] is not None and row[2] != -1 and row[2] != "" and row[2] != 0 and row[2] != None:
                stats[row[0]]["years"].append(row[2])
        else:
            stats[row[0]]={}
            stats[row[0]]["SEVERITY"] = []
            stats[row[0]]["years"] = []
            if row[1] is not None and row[1] != -1 and row[1] != "" and row[1] != 0 and row[1] != None:
                stats[row[0]]["SEVERITY"].append(row[1])
            if row[2] is not None and row[2] != -1 and row[2] != "" and row[2] != 0 and row[2] != None:
                stats[row[0]]["years"].append(row[2])
        rows.updateRow(row)
del rows    

##Go through table and populate the new fields with appropriate data from the dictionary above
with arcpy.da.UpdateCursor("dmg_attributes",["dmg_duplicates_FEAT_SEQ","SUM_SEVERITY","MEAN_SEVERITY","START_YR","END_YR","AVG_YRS_BTWN_DMG"]) as rows:
    for row in rows:
        sev=0
        for s in stats[row[0]]["SEVERITY"]:
            sev+=s
        row[1]=sev
        if len(stats[row[0]]["SEVERITY"]) != 0:
            row[2]=sev/len(stats[row[0]]["SEVERITY"])
        yrs=[]
        for y in range (1, len(stats[row[0]]["years"])):
            yrs.append(abs(stats[row[0]]["years"][y]-stats[row[0]]["years"][y-1]))
        ys = 0
        for yr in yrs:
            ys+=yr
        if stats[row[0]]["years"]:
            row[3] = min(stats[row[0]]["years"])
            row[4] = max(stats[row[0]]["years"])
        if len(yrs) != 0:
            row[5]= ys/len(yrs)
        rows.updateRow(row)
                    
# delete the cursor to ensure that temporary geodatabase can be deleted                    
del rows    

##delete the duplicate polygons using the unique polygon ID and then rejoin the attribute data from the original shapefile
arcpy.DeleteIdentical_management("dmg_attributes","dmg_duplicates_FEAT_SEQ")

##do a spatial join to get damage occurances, add a new field to hold occurance values, populate it from the join count field and then delete the join count field. 
arcpy.SpatialJoin_analysis("dmg_attributes","damaged_areas",dmg_type+"_"+dmg_agnt, "JOIN_ONE_TO_ONE", "", "", "WITHIN");
arcpy.AddField_management(dmg_type+"_"+dmg_agnt, "DMG_OCC", "SHORT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "");
arcpy.CalculateField_management(dmg_type+"_"+dmg_agnt, "DMG_OCC", '!Join_Count!','PYTHON');
arcpy.DeleteField_management(dmg_type+"_"+dmg_agnt, "Join_Count");

## Add a field for the type of map being generated (used by Carto)
arcpy.AddField_management(dmg_type+"_"+dmg_agnt, "mapname", "TEXT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "");
## convert type to that expected by Carto
if dmg_type=="AllTypes": 
	mapdmg_type="All"
elif dmg_type=="Mortality":
	mapdmg_type="Mort"
elif dmg_type=="Defoliation":
	mapdmg_type="Defo"
else:
	mapdmg_type=dmg_type

if dmg_agnt=="AllAgents":
	mapdmg_agnt="All"
else:
	mapdmg_agnt=dmg_agnt
expression="mapname()"
code="def mapname():\n    return '"+ mapdmg_agnt+"_"+mapdmg_type+"'";
arcpy.CalculateField_management(dmg_type+"_"+dmg_agnt, "mapname", expression, 'PYTHON',code);
##add damage occurrence and mapname to list of fields to keep
fields.append("DMG_OCC")
fields.append("mapname")

##make list of fields to delete. These are repeats of fields that already exist. 
deletefields = []
for fds in arcpy.ListFields(dmg_type+"_"+dmg_agnt,"*"):
    if fds.name not in fields:
        deletefields.append(fds.name)
        
## delete duplicate fields
arcpy.DeleteField_management(dmg_type+"_"+dmg_agnt, deletefields)

##convert feature class to shapefile and save
arcpy.FeatureClassToShapefile_conversion(dmg_type+"_"+dmg_agnt,folder);
arcpy.Delete_management(temp);
